package classicaltm.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.Timer;

import classicaltm.model.TuringMachine;

/**
 * A panel in which the user can actually simulate the Turing machine.
 */
public class SimulatorTab extends JPanel {
	
	/**
	 * The machine that is simulated. This may not be <code>null</code>
	 * whenever the tab is shown.
	 */
	private TuringMachine machine;
    
    /**
     * The toolbar for the {@link SimulatorMode#ASKING_INPUT} mode.
     */
    InputToolBar inputToolBar;
	
	/**
	 * The toolbar for the {@link SimulatorMode#SIMULATING} mode.
	 */
	SimulatingToolBar simulatingToolBar;
    
    /**
     * The toolbar for the {@link SimulatorMode#SHOWING_OUTPUT} mode.
     */
    OutputToolBar outputToolBar;
	
	/**
	 * The program panel of the simulator tab.
	 */
	private TextProgramPanel programPanel;
	
	/**
	 * The tape panel of the simulator tab.
	 */
	private TapePanel tapePanel;
	
	/**
	 * Modes the simulator tab can be in.
	 */
	public enum SimulatorMode {
	    
	    /**
	     * The interface is asking the user for the input to place on the tape.
	     */
	    ASKING_INPUT,
	    
	    /**
	     * The interface is simulating the Turing program.
	     */
	    SIMULATING,
	    
	    /**
	     * The interface presents the output of the Turing program.
	     */
	    SHOWING_OUTPUT
	}
	
	/**
	 * The mode this simulator tab is in.
	 * 
	 * The mode influences the buttons / input fields in the toolbar in the tab.
	 */
	private SimulatorMode mode = SimulatorMode.ASKING_INPUT;
	
	/**
	 * The timer that performs steps automatically.
	 */
	private Timer t;
	
	/**
	 * Creates a new simulator tab.
	 */
	public SimulatorTab() {
		super(new BorderLayout());
		
		inputToolBar = new InputToolBar(this);
        simulatingToolBar = new SimulatingToolBar(this);
        outputToolBar = new OutputToolBar(this);
		
		programPanel = new TextProgramPanel();
		JPanel programPanelWrapper = new JPanel(new BorderLayout());
		programPanelWrapper.add(programPanel);
		programPanelWrapper.setBorder(BorderFactory.createTitledBorder("States"));
		add(programPanelWrapper, BorderLayout.CENTER);
		
		tapePanel = new TapePanel(null);
		JPanel tapePanelWrapper = new JPanel(new BorderLayout());
		tapePanelWrapper.add(tapePanel);
		tapePanelWrapper.setBorder(BorderFactory.createTitledBorder("Tape"));
		add(tapePanelWrapper, BorderLayout.PAGE_END);
		
		updateValues();
		
        t = new Timer(0, null);
        t.addActionListener(new ActionListener() {
            
            @Override
            public void actionPerformed(ActionEvent e2) {
                performStep();
            }
        });
	}
	
	/**
	 * Sets the Turing machine that is shown in the simulator tab.
	 * The values in the tab will also be updated.
	 * 
	 * @param machine The Turing machine.
	 */
	public void setTuringMachine(TuringMachine machine) {
		this.machine = machine;
		programPanel.setMachine(machine);
		tapePanel.setTape(machine.getTape());
		
		setMode(SimulatorMode.ASKING_INPUT);
		
		updateValues();
	}
	
	/**
	 * Returns the Turing machine that is shown in this tab.
	 * @return The Turing machine.
	 */
	public TuringMachine getTuringMachine() {
		return machine;
	}
	
	/**
	 * Updates the values in this tab. This should be called when the
	 * Turing machine changes its state (for example, when a step is executed).
	 */
	public void updateValues() {
		programPanel.updateValues();
		tapePanel.repaint();
	}
	
	/**
	 * Switches the simulator tab to another mode.
	 * 
	 * The GUI will be updated to reflect the new mode.
	 * 
	 * @param mode The new mode of this simulator tab.
	 */
    public void setMode(SimulatorMode mode) {
        this.mode = mode;
        
        Component currentToolBar = ((BorderLayout) getLayout()).getLayoutComponent(BorderLayout.PAGE_START);
        if (currentToolBar != null) {
            remove(currentToolBar);
        }
        
        switch (mode) {
        case ASKING_INPUT:
            add(inputToolBar, BorderLayout.PAGE_START);
            break;
            
        case SIMULATING:
            add(simulatingToolBar, BorderLayout.PAGE_START);
            break;
            
        case SHOWING_OUTPUT:
            add(outputToolBar, BorderLayout.PAGE_START);
        }
        
        invalidate();
        repaint();
    }
    
    /**
     * Returns the mode this tab is currently in.
     * @return The mode of this simulator tab.
     */
    public SimulatorMode getMode() {
        return mode;
    }
    
    /**
     * Executes one transition in the Turing machine. If the Turing machine is
     * finished, the simulator tab will switch to {@link SimulatorMode#SHOWING_OUTPUT}
     * mode.
     */
    public void performStep() {
        // perform a step
        getTuringMachine().step();
        
        // if we cannot do any transition, go to the output mode
        if (getTuringMachine().activeTransition() == null) {
            t.stop();
            simulatingToolBar.singleStepButton.setEnabled(true);
            simulatingToolBar.runUntilFinishedButton.setEnabled(true);
            simulatingToolBar.resetSimulationButton.setEnabled(true);
            
            ArrayList<Character> output = getTuringMachine().getTape().readOutput();
            String outputString = "";
            for (Character c : output) {
                outputString += c;
            }
            outputToolBar.setOutputText(outputString);
            setMode(SimulatorMode.SHOWING_OUTPUT);
        }
        
        // since the Turing machine has changed, we need to update the values shown
        updateValues();
    }
    
    /**
     * Causes the timer that fires transitions to be started.
     */
    public void startStepTimer() {
        simulatingToolBar.singleStepButton.setEnabled(false);
        simulatingToolBar.runUntilFinishedButton.setEnabled(false);
        simulatingToolBar.resetSimulationButton.setEnabled(false);
        t.start();
    }
    
    /**
     * Causes the timer that fires transitions to be stopped.
     */
    public void stopStepTimer() {
        simulatingToolBar.singleStepButton.setEnabled(true);
        simulatingToolBar.runUntilFinishedButton.setEnabled(true);
        simulatingToolBar.resetSimulationButton.setEnabled(true);
        t.stop();
    }
}
