package classicaltm.model;

import java.util.HashMap;
import java.util.HashSet;

/**
 * A state in the Turing machine.
 * 
 * A state maintains its outgoing transitions. States are identified in the
 * program by their names.
 */
public class State {
	
	/**
	 * The name of this state.
	 */
    private final String name;
    
    /**
     * A map of outgoing transitions from this state.
     * 
     * This hash map maps transition names to {@link Transition} objects.
     */
    private final HashMap<String, Transition> transitions;
    
    /**
     * Creates a new state with the given name, without outgoing transitions.
     * @param name The name of the state.
     */
    public State(String name) {
        this.name = name;
        this.transitions = new HashMap<>();
    }
    
    /**
     * Adds a new outgoing transition to this state.
     * 
     * @param t The transition to add.
     * @param tapeAlphabet The tape alphabet of the Turing machine.
     * This object is updated to include the tape symbols of this transition,
     * if that was not already the case.
     * @param states The set of states.
     * This object is updated to include the target state, if it did not include
     * that state already.
     * 
     * @throws IllegalStateException If the transition already exists among the
     * outgoing transitions of this state.
     * @throws NullPointerException If <code>t</code>, <code>tapeAlphabet</code>
     * or <code>states</code> was <code>null</code>.
     */
    public void addTransition(Transition t, HashSet<Character> tapeAlphabet,
    		HashMap<String, State> states) {
        
    	if (t == null || tapeAlphabet == null || states == null) {
    		throw new NullPointerException("t, tapeAlphabet and states may not be null");
    	}
    	
        if (transitions.containsKey(t.getTransitionId())) {
            throw new IllegalStateException("Transition " + t.getTransitionId() +
            		" already exists in state " + name);
        }
        
        // add the transition
		transitions.put(t.getTransitionId(), t);
		tapeAlphabet.add(t.getReadTape());
		tapeAlphabet.add(t.getWriteTape());
		
		if (!states.containsKey(t.getTargetState())) {
		    states.put(t.getTargetState(), new State(t.getTargetState()));
		}
    }
    
    /**
     * Returns the name of this state.
     * @return The state name.
     */
    public String getName() {
        return this.name;
    }
    
    /**
     * Returns a hash map of outgoing transitions of this state.
     * @return The outgoing transitions.
     */
    public HashMap<String, Transition> getOutgoingTransitions() {
        return this.transitions;
    }
}
