package classicaltm.model;

import java.util.HashMap;
import java.util.HashSet;

/**
 * A Turing program; that is, the sets of states and transitions that define the
 * functionality Turing machine.
 * 
 * A Turing program is represented by its set of states. The states, in turn, store
 * the outgoing transitions.
 * 
 * @see State
 * @see Transition
 */
public class TuringProgram {
	
	/**
	 * The states of the Turing program. This is a hash map that maps state names
	 * to states.
	 */
    private final HashMap<String, State> states;
    
    /**
     * The characters that can be put on the tape.
     */
    private final HashSet<Character> tapeAlphabet;
    
    /**
     * Creates a new Turing program without any states and transitions.
     */
    public TuringProgram() {
        this.states = new HashMap<>();
        this.tapeAlphabet = new HashSet<>();
    }
    
    /**
     * Adds a state with the given name to the Turing program.
     * @param s The name of the state to add.
     */
	public void addState(String s) {
		states.put(s, new State(s));
	} 
    
    /**
     * Adds a transition to the Turing program.
     * 
     * If the source state of the transition does not yet exist, it is added
     * as a new state to the Turing program.
     * 
     * @param t The transition to add.
     */
    public void addTransition(Transition t) {
    	
    	// if the source state is not known yet, add it
        if (!states.containsKey(t.getSourceState())) {
            states.put(t.getSourceState(), new State(t.getSourceState()));
        }
        states.get(t.getSourceState()).addTransition(t, tapeAlphabet, states);
    }
    
    /**
     * Returns a list of all states in this Turing program.
     * @return The states in this Turing program.
     */
    public HashMap<String, State> getStates() {
        return this.states;
    }
    
    /**
     * Returns the tape alphabet, that is, the set of all characters that can be
     * placed on the tape.
     * 
     * TODO [ws] research why this is needed :)
     * 
     * @return The tape alphabet.
     */
    public HashSet<Character> getTapeAlphabet() {
        return this.tapeAlphabet;
    }
    
    /**
     * Returns whether it is possible to run this Turing program.
     * 
     * A Turing program is runnable when it contains at least one state.
     * 
     * @return <code>true</code> if the program is runnable; <code>false</code>
     * otherwise.
     */
    public boolean isRunnable() {
        return !this.states.isEmpty();
    }
    
    /**
     * Returns a string containing all states.
     * 
     * TODO [ws] This may be more appropriate in the GUI instead of the model;
     * well, it doesn't hurt of course ;)
     * 
     * @return A string of the form "[state] \n [state] \n ... \n [state]".
     */
    public String getStateString() {
        
        String statesStr = "";
        
        for (String s : this.states.keySet()){
            statesStr += s + "\n";
        }
        return statesStr;
    }
    
    /**
     * Returns a string containing the tape alphabet.
     * 
     * TODO [ws] This may be more appropriate in the GUI instead of the model;
     * well, it doesn't hurt of course ;)
     * 
     * @return A string of the form "[char] \n [char] \n ... \n [char]".
     * @see #getTapeAlphabet()
     */
    public String getTapeAlphabetString() {
        
        String tapeAlphabetStr = "";
        
        for (char c : this.tapeAlphabet) {
            tapeAlphabetStr += c + "\n";
        }
        return tapeAlphabetStr;
    }
}
